
import java.sql.ResultSet;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.zaxxer.hikari.HikariDataSource;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;


public class JDBCTest {
      public static void main(String[] argv) throws Exception {
            Class.forName("org.postgresql.Driver");

            HikariDataSource dataSource = new HikariDataSource();

            dataSource.setPoolName("foo");
            dataSource.setJdbcUrl("jdbc:postgresql://127.0.0.1:5432/alexey?prepareThreshold=1");
            dataSource.setUsername("alexey");
            dataSource.setPassword("");
            dataSource.setMinimumIdle(0);
            dataSource.setMaximumPoolSize(1);
            dataSource.setIdleTimeout(10000 * 1000L);
            dataSource.setMaxLifetime(10000 * 1000L);
            JdbcTemplate jt = new JdbcTemplate(dataSource);

            int[][] paramsets = new int[][]{
                    {100000, 0, 0},
                    {1000, 10000, 1},
                    {1000, 1000, 1000},
                    {1000, 100, 10000},
                    {1000, 10, 100000},
                    {1000, 1, 1000000},
                    {10, 1, 100000000}
            };

            for(int[] paramset: paramsets) {

                int nruns = paramset[0];
                int nparams = paramset[1];
                int paramLength = paramset[2];

                String sql = "select coalesce('' "
                        + String.join("", Collections.nCopies(nparams, ", ?"))
                        + ")";
                String p = String.join("", Collections.nCopies(paramLength, "z"));
                Object[] pp = new Object[nparams];
                Arrays.fill(pp, p);

                RowMapper<String> rm = (ResultSet resultSet, int i) -> resultSet.getString(1);

                long startTime = System.nanoTime();

                for (int i = 0; i < nruns; i++) {
                    List<String> result = jt.query(sql, pp, rm);
                }

                long endTime = System.nanoTime();
                long duration = (endTime - startTime);
                System.out.println(String.format("NPARAMS=%d PARAM_LENGTH=%d NRUNS=%d TIME=%fs",
                        nparams, paramLength, nruns, duration / 1000000. / 1000));
            }
            System.exit(0);
      }
}
